/*
 * Copyright 2007 The authors
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.intellij.struts2.dom.struts.impl;

import com.intellij.psi.*;
import com.intellij.psi.util.PropertyUtil;
import com.intellij.struts2.dom.struts.action.Action;
import com.intellij.struts2.dom.struts.strutspackage.DefaultClassRef;
import com.intellij.struts2.dom.struts.strutspackage.StrutsPackage;
import com.intellij.util.xml.DomUtil;
import com.intellij.util.xml.GenericAttributeValue;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Adds utility methods.
 *
 * @author Yann Cbron
 */
public abstract class ActionImpl implements Action {

  @NotNull
  public StrutsPackage getStrutsPackage() {
    final StrutsPackage strutsPackage = DomUtil.getParentOfType(this, StrutsPackage.class, true);
    assert strutsPackage != null : "could not resolve enclosing <package> for " + this + " (" + this.getName().getStringValue() + ")";
    return strutsPackage;
  }

  @Nullable
  public PsiClass searchActionClass() {
    final GenericAttributeValue<PsiClass> actionClassAttribute = getActionClass();
    if (actionClassAttribute.getXmlElement() != null) {
      return actionClassAttribute.getValue();
    }
    // resolve parent package <default-class-ref> (walk upwards)
    final DefaultClassRef ref = getStrutsPackage().searchDefaultClassRef();
    if (ref != null) {
      return ref.getDefaultClass().getValue();
    }

    // nothing found in parents --> error highlighting
    return null;
  }

  @Nullable
  public PsiMethod searchActionMethod() {
    final GenericAttributeValue<PsiMethod> methodValue = getMethod();
    if (methodValue.getXmlElement() != null) {
      return methodValue.getValue();
    }

    final List<PsiMethod> methods = getActionMethods();
    for (final PsiMethod method : methods) {
      if (method.getName().equals("execute")) {
        return method;
      }
    }

    return null;
  }

  @Nullable
  public String getNamespace() {
    return getStrutsPackage().getNamespace().getStringValue();
  }

  @NotNull
  public List<PsiMethod> getActionMethods() {
    final PsiClass actionClass = getActionClass().getValue();
    if (actionClass == null) {
      return Collections.emptyList();
    }

    final List<PsiMethod> actionMethods = new ArrayList<PsiMethod>();
    for (final PsiMethod psiMethod : actionClass.getAllMethods()) {
      // only public non-static concrete methods
      final PsiModifierList modifiers = psiMethod.getModifierList();
      if (!modifiers.hasModifierProperty(PsiModifier.PUBLIC) ||
          modifiers.hasModifierProperty(PsiModifier.STATIC) ||
          modifiers.hasModifierProperty(PsiModifier.ABSTRACT)) {
        continue;
      }

      // no parameters
      if (psiMethod.getParameterList().getParametersCount() != 0) {
        continue;
      }

      if (psiMethod.getName().equals("toString")) {
        continue;
      }

      // do not include "public String getXXX()" methods
      if (PropertyUtil.isSimplePropertyGetter(psiMethod)) {
        continue;
      }

      // "throws java.lang.Exception" or nothing
      final PsiClassType[] exceptionTypes = psiMethod.getThrowsList().getReferencedTypes();
      if (exceptionTypes.length > 1) {
        continue;
      }

      if (exceptionTypes.length == 1) {
        final String exceptionClassName = exceptionTypes[0].getCanonicalText();
        if (!exceptionClassName.equals("java.lang.Exception")) {
          continue;
        }
      }

      final PsiType type = psiMethod.getReturnType();
      if (type != null &&
          type instanceof PsiClassType &&
          type.getCanonicalText().equals(CommonClassNames.JAVA_LANG_STRING)) {
        actionMethods.add(psiMethod);
      }
    }
    return actionMethods;
  }
}