/*
 * Copyright 2007 The authors
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.intellij.struts2.reference;

import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiReference;
import com.intellij.psi.PsiReferenceBase;
import com.intellij.psi.impl.source.resolve.reference.PsiReferenceProviderBase;
import com.intellij.psi.xml.XmlAttributeValue;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

/**
 * Provides reference functionality for static String values.
 *
 * @author Yann Cbron
 */
public class StaticStringValuesReferenceProvider extends PsiReferenceProviderBase {

  private final boolean soft;
  private final String[] values;


  /**
   * Creates a reference provider with the given values for autocompletion.
   * Other values will *not* be highlighted as errors.
   *
   * @param values Autocompletion values.
   */
  public StaticStringValuesReferenceProvider(@NotNull @NonNls final String... values) {
    this(true, values);
  }

  /**
   * Creates a reference provider with the given values for autocompletion and optional error highlighting.
   *
   * @param allowOtherValues Set to false to enable error highlighting.
   * @param values           Autocompletion values.
   */
  public StaticStringValuesReferenceProvider(final boolean allowOtherValues, @NotNull @NonNls final String... values) {
    this.soft = allowOtherValues;
    this.values = values;
  }

  @NotNull
  public PsiReference[] getReferencesByElement(final PsiElement psiElement) {
    return new PsiReference[]{new PsiReferenceBase<XmlAttributeValue>((XmlAttributeValue) psiElement) {
      public PsiElement resolve() {
        if (soft || myElement.getValue().startsWith("%{")) {
          return myElement;
        }

        for (final String value : values) {
          if (myElement.getValue().equals(value)) {
            return myElement;
          }
        }
        return null;
      }

      public Object[] getVariants() {
        return values;
      }
    }};
  }

}